from ldnode.calculator.schemas import ComputedFactor
from .common import TaskBaseParameters


class FormulaComputeParameters(TaskBaseParameters):
    """ 可转债公式计算参数 """
    formula_list: list[list[ComputedFactor]]
