from .factor import ProcessMissingDataMethod, NeutralizationMethod
from .screen import BuiltinScope


class ETFNeutralizationMethod(NeutralizationMethod):
    """ ETF中性化方法 """


class ETFProcessMissingDataMethod(ProcessMissingDataMethod):
    """ ETF缺失值处理方式 """


class ETFBuiltinScope(BuiltinScope):
    """基金内置交易所板块"""
