import polars as pl


def plan_rename(lf: pl.LazyFrame) -> pl.LazyFrame:
    return lf.rename({
        'close': 'pre_close',
        'turnover': 'vol',
        'volume': 'amount',
        'date': 'trade_date'
    })
    