from functools import lru_cache

from .settings import Settings


@lru_cache()
def get_settings() -> Settings:
    """
        获取配置
        @return: 配置
    """
    return Settings()


__all__ = ['get_settings', "Settings"]
