from clickhouse_driver import Client

from ldnode.runner.config import get_settings


settings = get_settings()


def get_clickhouse_client() -> Client:
    """
        获取Clickhouse客户端

        :return: Clickhouse客户端
        :rtype: clickhouse_driver.Client
    """
    return Client.from_url(settings.CLICKHOUSE_URI)
