from .schemas.models import File

TRADE_DAYS = File(key="dim/trade_days.parquet")  # 交易日历
INDEX_DATA = File(key="backtest_data/index.parquet")  # 回测基准指数

CB_DATA = File(key="cb_data.pq")  # 转债回测数据
CB_PRE_QUOTATION = File(key="before_trading/pre_quotations.pq")  # 转债盘前数据
CB_HIST_DATA = File(key="before_trading/hist_data.pq")  # 转债历史行情
CB_QUOTATION = File(key="cb_quotations.pq")  # 转债实时截面

STK_DATA = File(key="stk_data.pq")  # 股票回测数据
STK_PRE_QUOTATION = File(key="before_trading/stk_pre_quotations.pq")  # 股票盘前数据
STK_HIST_DATA = File(key="before_trading/hist_data_stk.pq")  # 股票历史行情
STK_QUOTATION = File(key="stk_quotations.pq")  # 股票实时截面

ETF_DATA = File(key="backtest_data/etf.parquet")  # ETF回测数据
ETF_PRE_QUOTATION = File(key="before_trading/etf.parquet")  # ETF盘前数据
ETF_QUOTATION = File(key="etf_quotations.pq")  # ETF实时截面
