CB_SCREEN_RESULT_FIXED_COLUMNS: list[str] = [
    'code', 'name', 'time', 'rank', '_weight', 'selected', 'close', 'pct_chg', 'amount', 'limit',
]


STK_SCREEN_RESULT_FIXED_COLUMNS: list[str] = [
    'code', 'name', 'time', 'rank', '_weight', 'selected', 'close', 'pct_chg', 'amount', 'limit',
]


ETF_SCREEN_RESULT_FIXED_COLUMNS: list[str] = [
    'code', 'name', 'time', 'rank', '_weight', 'selected', 'close', 'pct_chg', 'amount', 'limit',
]
""" ETF截面结果固定列 """
